/**
    This class models circles in the plane.

   This is an immutable version of Circle.
*/

public class Circle
{
   // Declare three instance variables for a Circle object.
   // Make them private instance variables.
   private double xCenter;
   private double yCenter;
   private double radius;

   /**
       Define a constructor for initializing this circle's instance variables.
   */
   public Circle(double x, double y, double r)
   {
      xCenter = x;
      yCenter = y;
      radius = r;  // what should we do about a negative radius?
   }

   /**
       An accessor method to return the radius.
   */
   public double getRadius()
   {
      return radius;
   }

   /**
       An accessor method to return the x-value of the center.
   */
   public double getX()
   {
      return xCenter;
   }

   /**
       An accessor method to return the y-value of the center.
   */
   public double getY()
   {
      return yCenter;
   }

   /**
       A method to compute this circle's area.
   */
   public double computeArea()
   {
      return Math.PI*Math.pow(radius,2);
   }

   /**
       A method to compute this cirlcle's perimeter.
   */
   public double computePerimeter()
   {
      return 2*Math.PI*radius;
   }

   /*
      NOTE: To make this class "immutable" we change all the
      mutator methods to be methods that return a new Circle
      object that holds the mutated vlaue from this circle.
   */

   /**
       Return a new Circle equivalent to this circle but moved by the given amount.
   */
   public Circle moveBy(double x, double y)
   {
      return new Circle(xCenter + x, yCenter + y, radius);
   }

   /**
       Return a new Circle equivalent to this circle but moved up (or down).
   */
   public Circle moveUp(double distance)
   {
      return new Circle( xCenter + distance, yCenter, radius);
   }

   /**
       Return a new Circle equivalent to this circle but moved right (or left).
   */
   public Circle moveRight(double distance)
   {
      return new Circle(xCenter, yCenter + distance, radius);
   }

   /**
       Return a new Circle equivalent to this circle but with a new radius.
   */
   public Circle grow(double change)
   {
      return new Circle(xCenter, yCenter, radius + change);
   }

   /**
       A method to determine if this circle
       intersects (overlaps) another circle.
   */
   public boolean intersect(Circle circle2)
   {
      // compute the distance between the centers
      double deltaX = this.xCenter - circle2.xCenter;
      double deltaY = this.yCenter - circle2.yCenter;
      double distance = Math.sqrt( deltaX*deltaX + deltaY*deltaY );

      return ( distance < (this.radius + circle2.radius) );
   }

   /**
       A method to determine if this circle
       is equal to another circle.
   */
   public boolean equals(Circle circle2)
   {
      return ( (this.xCenter == circle2.xCenter)
            && (this.yCenter == circle2.yCenter)
            && (this.radius  == circle2.radius  ));
   }

   /**
       A method to print out this Circle object's data.
   */
   public void writeOutput()
   {
      System.out.println("This circle has the following properties:");
      System.out.println("\tcenter = (" + xCenter + "," + yCenter + ")" );
      System.out.println("\tradius  = " + radius);
   }

}//Circle
