/**
   This class illustrates overloading.

   You can overload a method name by changing either
   the type of a parameter, or the number of parameters.

   Compare with
      https://docs.oracle.com/javase/8/docs/api/java/lang/Math.html
   or
      https://docs.oracle.com/javase/8/docs/api/java/util/Arrays.html
*/
public class Max_version_1
{
   public static double max(double x1, double x2)
   {
      double result = x1;
      if (x2 > x1)
         result = x2;
      return result;
   }

   // Overload by number of arguments.
   public static double max(double x1, double x2, double x3)
   {
      double result = x1;
      if (x2 > result)
         result = x2;
      if (x3 > result)
         result = x3;
      return result;
   }

   // Overload by number of arguments.
   public static double max(double x1, double x2, double x3, double x4)
   {
      double result = x1;
      if (x2 > result)
         result = x2;
      if (x3 > result)
         result = x3;
      if (x4 > result)
         result = x4;
      return result;
   }

   // Overload by number of arguments.
   public static double max(double x1, double x2, double x3, double x4, double x5)
   {
      double result = x1;
      if (x2 > result)
         result = x2;
      if (x3 > result)
         result = x3;
      if (x4 > result)
         result = x4;
      if (x5 > result)
         result = x5;
      return result;
   }

   // An overloaded method that takes an arbitrary number of parameters
   public static double max(double... x) // notice the "dot dot dot"
   {
      double result = x[0];  // the parameter x is really an array
      for (int i = 1; i < x.length; i++)
      {
         if (x[i] > result)
            result = x[i];
      }
      return result;
   }

    // Overload by type of argument.
    public static int max(int... x)
    {
       int result = x[0];
       for (int i = 1; i < x.length; i++)
       {
          if (x[i] > result)
             result = x[i];
       }
       return result;
    }


    public static void main(String[] args)
    {
        double m1 = max(4.0, 5.0);
        double m2 = max(1.0, 2.0, 3.0);
        double m3 = max(3.1, 2.4, 3.6, 7.8, 8.2, 9.1, 5.6, 6.2, 7.4);
           int m4 = max(3, 5, 2, 8, 6, 0, 2, 5, 7, 2, 8);

        System.out.println("m1 = " + m1);
        System.out.println("m2 = " + m2);
        System.out.println("m3 = " + m3);
        System.out.println("m4 = " + m4);
    }
}
