/**
   This class illustrates overloading.

   You can overload a method name by changing either
   the type of a parameter, or the number of parameters.

   Compare with
      https://docs.oracle.com/javase/8/docs/api/java/lang/Math.html
   or
      https://docs.oracle.com/javase/8/docs/api/java/util/Arrays.html
*/
public class Max_version_2
{
   public static double max(double x1, double x2)
   {
      double result = x1;
      if (x2 > x1)
         result = x2;
      return result;
   }

   // Overload by number of arguments.
   public static double max(double x1, double x2, double x3)
   {
      return max( x1, max(x2, x3));
   }

   // Overload by number of arguments.
   public static double max(double x1, double x2, double x3, double x4)
   {
      return max( x1, max(x2, x3, x4));
   }

   // Overload by number of arguments.
   public static double max(double x1, double x2, double x3, double x4, double x5)
   {
      return max( x1, max(x2, x3, x4, x5));
   }

   public static void main(String[] args)
   {
      double m1 = max(4.0, 5.0);
      double m2 = max(1.0, 2.0, 3.0);
      double m3 = max(3.1, 2.4, 3.6, 7.8, 8.2);

      System.out.println("m1 = " + m1);
      System.out.println("m2 = " + m2);
      System.out.println("m3 = " + m3);
   }
}
